/* 
 * File:   CAN_vars.h
 * Author: charl
 *
 * Created on March 14, 2021, 4:53 PM
 */

#ifndef CAN_VARS_H
#define	CAN_VARS_H

//########################### CAN Variables ###########################//    

/* Data structure for CAN full receive message buffer.*/
    
/* Define the sub-components of the data structure as specified in Table 34-5 */
/* Create a CMSGSID data type. */
    typedef struct
    {
        unsigned SID:11;
        unsigned FILHIT:5;
        unsigned CMSGTS:16;
    }rxcmsgsid;
/* Create a CMSGEID data type. */
    typedef struct
    {
        unsigned DLC:4;
        unsigned RB0:1;
        unsigned :3;
        unsigned RB1:1;
        unsigned RTR:1;
        unsigned EID:18;
        unsigned IDE:1;
        unsigned SRR:1;
        unsigned :2;
    }rxcmsgeid;
/* Create a CMSGDATA0 data type. */
    typedef struct
    {
        unsigned Byte0:8;
        unsigned Byte1:8;
        unsigned Byte2:8;
        unsigned Byte3:8;
    }rxcmsgdata0;
/* Create a CMSGDATA1 data type. */
    typedef struct
    {
        unsigned Byte4:8;
        unsigned Byte5:8;
        unsigned Byte6:8;
        unsigned Byte7:8;
    }rxcmsgdata1;
    
/* This is the main data structure. */
    typedef union uCANRxMessageBuffer {
    struct
        {
            rxcmsgsid CMSGSID;
            rxcmsgeid CMSGEID;
            rxcmsgdata0 CMSGDATA0;
            rxcmsgdata0 CMSGDATA1;
    };
        int messageWord[4];
    }CANRxMessageBuffer;
    
   volatile int EID = 0;
     float rpm_R = 0;
     float rpm_L = 0;
     float speed = 0;
     float power = 0;
     float motor_current_R = 0;
     float motor_current_L = 0;
     int bat_voltage_R = 0;
     int error_lsb_R = 0;
     int error_msb_R = 0;
     int bat_voltage_L = 0;
     int error_lsb_L = 0;
     int error_msb_L = 0;
    volatile int contr_temp_R = 0;
    volatile int contr_temp_L = 0;
    volatile int contr_temp_G = 0;
    volatile int contr_temp_LR = 0;
     int motor_temp_R = 0;
    volatile int motor_temp_L = 0;
    volatile int mot_temp_G = 0;
    volatile int mot_temp_LR = 0;
    volatile int contr_status_R = 0;
    volatile int contr_status_L = 0;
    volatile int switch_status_R = 0;
    volatile int switch_status_L = 0;
    volatile float throttle_R = 0;
    volatile float throttle_L = 0;
    
    volatile char* msg_array[8];
    
    // define CAN output
    struct CAN_out0_R {
    float   rpm_R,  motor_current_R;
    int bat_voltage_R, error_lsb_R,  error_msb_R;
        };
    
    struct CAN_out1_R {
    float throttle_R;
    int contr_temp_R, motor_temp_R, contr_status_R, switch_status_R;
    };
    
    struct CAN_out0_L {
    float   rpm_L,  motor_current_L;
    int bat_voltage_L, error_lsb_L,  error_msb_L;
        };
    
    struct CAN_out1_L {
    float throttle_L;
    int contr_temp_L, motor_temp_L, contr_status_L, switch_status_L;
    };
    
    struct CAN_out {
    float   rpm_R,  motor_current_R, throttle_R, rpm_L,  motor_current_L, throttle_L;
    int bat_voltage_R, error_lsb_R,  error_msb_R, bat_voltage_L, error_lsb_L, error_msb_L, contr_temp_R, motor_temp_R, contr_status_R, switch_status_R, contr_temp_L, motor_temp_L, contr_status_L, switch_status_L;
        };
   
#endif	/* CAN_VARS_H */

