#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <xc.h>
#include <sys/attribs.h>
#include <sys/kmem.h>
#include "FT8xx.h"
#include "Library.h"
#include "function_prototypes.h"

void initIO(void){
    
    //TRISEbits.TRISE7 = 0;   // Rotary Encoder CS set as output
    TRISEbits.TRISE6 = 0;   // GEN DAC CS set as output
    TRISGbits.TRISG9 = 0;   // Left DAC CS set as output
    TRISFbits.TRISF3 = 0;   // Right DAC CS set as output
    
    TRISBbits.TRISB2 = 1;       // configure analog pin RB2 (direction_in) as input
    TRISEbits.TRISE3 = 0;       // configure digital pin RE3 (direction_out) as output
    TRISCbits.TRISC13 = 0;      // configure digital pin RC13 (AIR_interrupt) as output
    //TRISBbits.TRISB4 = 1;       // configure pin RB4 (throttle1) as input
//    TRISBbits.TRISB5 = 1;       // configure pin RB5 (throttle2) as input
    TRISBbits.TRISB3 = 1;       // configure pin RB3 (brake) as input
    TRISFbits.TRISF5 = 0;
    LATFbits.LATF5 = 1;
    
    TRISFbits.TRISF1 = 0;
    LATFbits.LATF1 = 0;
    
    TRISDbits.TRISD7 = 1;       // configure pin RD7 (Enable) as input
    TRISDbits.TRISD6 = 1;       // configure pin RD6 (Charge Enable) as input
    TRISEbits.TRISE5 = 0;
    TRISDbits.TRISD9 = 0;

    
    // Set all inputs to digital //
    AD1PCFG = 0xFFFF;

    // Enable pull up resistors //
    CNPUEbits.CNPUE15 = 1;  // configure pull up resistor for ENABLE pin
    CNPUEbits.CNPUE16 = 1;  // configure pull up resistor for CHARGE_ENABLE pin
 
    //configure ignition 
    TRISBbits.TRISB6 = 0;    

}

//
// Interrupts definitions //
//

void initInterrupts(void){
    INTCONbits.MVEC = 1;        // enable multivectored interrupts
    asm("ei");                  // enable interrupts
    initTimer3();               // enable interrupts for CAN bus
    initTimer5();
   

    
// initialize timer for Rotary Encoder
//    T5CONbits.ON = 1;           // turn on timer this might be a problem(as we
    //are out of timers. Might need to combine them)
}

//change interrupt for Engine Feedback Loop
void initINT1(void) {
    IEC0bits.INT1IE = 0;
    IFS0bits.INT1IF = 0;
    //IPC1bits.INT1IP = 1;
    IPC1bits.INT1IP = 3;
    INTCONbits.INT1EP = 1;
    IEC0bits.INT1IE = 1;
}

    // Timer 3 for CAN interface //
void initTimer3(void){
    T3CONbits.ON = 0; // turn off timer3
    // Initialize timer 3 interrupt bits //
    IEC0bits.T3IE = 1; // enable timer3 interrupt
    IPC3bits.T3IP = 6; //int priority -- 4

    // Enable timer 3 //
    T3CONbits.ON = 0; // turn it off
    TMR3 = 0; // clear any count
    
    /* Set timer length
     * PR*PreScale = 64*500000 = 320,000,00
     * 80Mhz/(PR*PreScale) = 2.5 Hz */
    PR3 = 5000000;
    T3CONbits.TCKPS = 0b110; // divide by 64

}

// Timer 5 for Nextion
void initTimer5(void){
    T5CONbits.ON = 0; // turn off timer5
    // Initialize timer 5 interrupt bits //
    IEC0bits.T5IE = 1; // enable timer5 interrupt
    IPC5bits.T5IP = 0b101; //int priority -- 5

    // Enable timer 5 //
    T5CONbits.ON = 0; // turn it off
    TMR5 = 0; // clear any count
    
    /* Set timer length
     * PR*PreScale = 64*1250000 = 80000000
     * 80Mhz/(PR*PreScale) = 1 Hz */
    PR5 = 2500000;
    T5CONbits.TCKPS = 0b110; // divide by 64
}

//Timer 4 for Engine Feedback Loop
void initTimer4(void) {
    T4CONbits.ON = 0; //turn off Timer 4
    TMR4 = 0; // clear count
    IEC0bits.T4IE = 1; // enable timer2 interrupt
    IPC4bits.T4IP = 4; //int priority -- 6
   
    /* Set timer length
     * PR*PreScale = 32*500000 = 16000000
     * 80Mhz/(PR*PreScale) = 5 Hz */
    PR4 = 500000;
    T4CONbits.TCKPS = 0b101; // divide by 32
}

//OC for engine feedback loop output
void initOC(void) {
    T2CONbits.TCKPS = 0b010; //set prescale to 4
    
    OC1CON = 0x0000; //Turn off OC1
    OC1CONbits.OCM = 0b110; //Configure for PWM mode without Fault pin enabled
    OC1CONbits.OC32 = 0;
    OC1CONbits.OCTSEL = 0;
    PR2 = 49999; // Set period using Timer 2
    
    OC1RS = 5300; // Initialize secondary Compare register //figure out a way to not set this value here this here
    OC1R = 5300; //Initialize primary Compare resister
    OC1CONbits.OCM = 0b110; //Configure for PWM mode without Fault pin enabled
    OC1CONbits.OC32 = 0;
    OC1CONbits.OCTSEL = 0;
   
    T2CONbits.ON = 1; //Enable timer 2
    OC1CONbits.ON = 1; // Enable OC1
}

