#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <xc.h>
#include <sys/attribs.h>
#include <sys/kmem.h>
#include "FT8xx.h"
#include "Library.h"
#include "function_prototypes.h"
#include "nextion_var.h"
#include "global.h"
#include "SPI.h"


void configUART2(void) {
    U2MODEbits.ON = 0;          // be sure it is off
    U2MODEbits.BRGH = 1;        // high speed mode
    U2BRG = 79;                // store value in register  259 for 9600, 129 for 19200, 79 for 31250
    
    U2MODEbits.PDSEL = 0;       // 8 bit data, no parity
    U2MODEbits.STSEL = 0;       // 1 stop bit
    
    U2MODEbits.UEN = 0b00;      // UxTX and UxRX pins are enabled and used; no flow control pins
    U2STAbits.UTXEN = 1;        // enable transmitter
    U2STAbits.URXEN = 1;        // enable receiver
    U2MODEbits.ON = 1;          // turn it on
    
    right_error = "";
    left_error = "";
    
}

#define buffer_full U2STAbits.UTXBF // buffer full
void tputc(unsigned char val) {
    while (buffer_full); // wait if buffer full
    U2TXREG = val; // send character
    
}

#define data_avail U2STAbits.URXDA // data avail
unsigned char tgetc(void) {
    unsigned char val;
    while (!data_avail); // wait for character to appear
    val = U2RXREG; // get character
    tputc(val); // echo
    return val;
}



void _mon_putc(char c) {
    tputc(c);
}

/******* Function for communication with Nextion  ***********/

// Function that sends each character individually to the display
void Send(unsigned char *txt){
    short i = strlen(txt);
    
    while(1){
        
        tputc(*txt);
        ++txt;
        --i;
        if(i==0){
            break;
        }  
    }   
}


unsigned char Serial_Write(unsigned char c)
{
    tputc(c);
    return 1;
}

// Function that is required at the end of each command
void Terminate(void) {
    Serial_Write(0xFF);
    Serial_Write(0xFF);
    Serial_Write(0xFF);
}

// Function displays the rpm number on the screen and displays car speed
void Write_val(int rpm, int speed){
    if (rpm > 0){
        Terminate();
        Send("nrpm.val=");
        printf("%d",rpm);
        Terminate();
        Send("nspeed.val=");
        printf("%d",speed);
        Terminate();
    }
}

// Function updates the speed and rpm gauge
void Write_gauge(int rpm, int speed){
    int rpm_gauge = ((rpm * (33))) ;       //mapping function for the gauge
    if(rpm_gauge > 359){
        rpm_gauge = rpm_gauge-360;
    }
    int speed_gauge = ((speed * (135.0/50.0)));
    if(speed_gauge > 359){
        speed_gauge = speed_gauge-360;
    }
    Terminate();
    Send("nrpm.val=");
    printf("%d",rpm*10);
    Terminate();
    Send("zrpm.val=");
    printf("%d",rpm*10);
    Terminate();
    Send("nspeed.val=");
    printf("%d",speed_gauge*10);
    Terminate();  
}

void Write_Page2(int rpm,int rpm_left,int bat_volt){
    if (rpm > 0){
        Send("nr_rpm.val=");
        printf("%d", rpm*10);
        Terminate();
        Send("nl_rpm.val=");
        printf("%d", rpm*10);
        Terminate();
    }
    if (bat_volt > 0)
    {
      /*Send("jvolt.val=");
      printf("%d", 20);//bat_volt*0.833);
      Terminate();
      Send("npercentv.val=");
      printf("%d",20); //bat_volt*0.833); 
      Terminate();*/
      Send("nc_volt.val=");
      printf("%d",bat_volt);
      Terminate();  
    }
    
   
}


// Display the capacitor voltage on the Nextion
void Write_voltage(int bat_volt){
    Terminate();
    Send("jvolt.val=");
    printf("%d",bat_volt);
    Terminate();
    Send("npercentv.val=");
    printf("%d",bat_volt); 
    Terminate();
}

// Display the error messages on page 2 of the nextion
void Write_error(char* left, char* right){
    Terminate();
    Send("t5.txt=\"");
    Send(left);
    Send("\"");
    Terminate();
    Send("t6.txt=\"");    //not sure if these are the correct names on display
    Send(right);
    Send("\"");
    Terminate();
    Send("t7.txt=\"");
    Send(AMSboard);
    Send("\"");
    Terminate();
}

// Decode the error messages from the CAN for the right kelly (from 2018 code)
void decodeErrors_right(void) {
    
    uint16_t error_msg = (error_msb_right << 8) | (error_lsb_right & 0xff);
    int i=0;
    int comparison = 0b00000001;
    char *Error_messages[] = {"ID Angle", 
                                "Over Voltage", 
                                "Low Voltage",
                                "Unknown Error",
                                "Stall", 
                                "Internal Volts Fault",
                                "Controller temp",
                                "H Pedal",
                                "Unknown Error",
                                "Reset",
                                "Throttle Fault",
                                "Angle sensor",
                                "Unknown Error",
                                "Unknown Error",
                                "Motor Temp",
                                "Hall Galvanometer"};


    char* messages = " "; 
    new_CAN_Error_MSG_right = 0;
    CAN_COUNT_right=0;
    for(i=0;i<16; i++){
        if((error_msg & comparison) > 0){
            errorPointer_right[CAN_COUNT_right] = Error_messages[i];
            CAN_COUNT_right++;
            new_CAN_Error_MSG_right = 1;
        }
        comparison = comparison << 1;
    }
}

// Decode errors from left kelly
void decodeErrors_left(void) {
    uint16_t error_msg = (error_msb_left << 8) | (error_lsb_left & 0xff);
    int i=0;
    int comparison = 0b00000001;
    char *Error_messages[] = {"ID Angle", 
                                "Over Voltage", 
                                "Low Voltage",
                                "Unknown Error",
                                "Stall", 
                                "Internal Volts Fault",
                                "Controller temp",
                                "H Pedal",
                                "Unknown Error",
                                "Reset",
                                "Throttle Fault",
                                "Angle sensor",
                                "Unknown Error",
                                "Unknown Error",
                                "Motor Temp",
                                "Hall Galvanometer"};


    char* messages = " "; 
    new_CAN_Error_MSG_left = 0;
    CAN_COUNT_left=0;
    //errorPointer_left[CAN_COUNT_left] = Error_messages[5];
    for(i=0;i<16; i++){
        if((error_msg & comparison) > 0){
            errorPointer_left[CAN_COUNT_left] = Error_messages[i];
            CAN_COUNT_left++;
            new_CAN_Error_MSG_left = 1;
        }
        comparison = comparison << 1;
    } 
}


/* Timer 5 Interrupt Service Routine */
/* vector number for T2 is 20, Priority 5 */
void __ISR(20, IPL6AUTO) Timer5Hand(void){
    T5CONbits.ON = 0; // turn off timer2
    //int rpm2;
    //int rpm_previous;
    //right_error_previous = right_error;
    //left_error_previous = left_error;
    decodeErrors_right();
    decodeErrors_left();
    if (new_CAN_Error_MSG_right == 1) {
    right_error = errorPointer_right[CAN_COUNT_right-1];
    }
    else { 
       right_error = "No Error";
    }
    if (new_CAN_Error_MSG_left == 1) {
       left_error = errorPointer_left[CAN_COUNT_left-1];
    }
    else {
          left_error = "No Error";
    }
   

    motor_rpm = rpm;
    speed_car = rpm *21.29*3.14159*(60 / 63360);
    rpm_gauge = rpm / 100;
    
    // Write to Nextion
    
    Write_voltage(bat_volt);
    Write_Page2(rpm,rpm_left,bat_volt);
    Write_val(motor_rpm,speed_car);
    Write_gauge(rpm_gauge,speed_car);
    Write_error(left_error,right_error);
    

    IFS0bits.T5IF = 0; // clear interrupt flag
}
